﻿// ----------------------------------------------------------------------------
// <copyright company="F.C. Soft., Inc.">
//   Copyright(c) F.C. Soft., Inc.  All rights reserved.
// </copyright>
// <license>
//   Licensed to F.C. Soft., Inc. (FCSoft) under one or more contributor
//   license agreements.  See the NOTICE file distributed with this work for
//   additional information regarding copyright ownership.  FCSoft licenses
//   this file to You under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.  You
//   may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
// </license>
// ----------------------------------------------------------------------------
namespace FCSoft.SilverFrost.Framework
{
    using NUnit.Framework;


    /// <summary>
    /// <see cref="ArrayUtility"/> をテストします。
    /// </summary>
    [TestFixture]
    public class ArrayUtilityTest
    {
        /// <summary>
        /// <see cref="TestEqualsByte"/> のテストデータです。
        /// </summary>
        private static readonly TestCaseData[] EqualsByteTestData =
        {
            // 長さ 0
            new TestCaseData(
                null,
                0,
                null,
                0,
                0)
                .Returns(true)
                .SetName("A01: Equals(null, 0, null, 0, 0)"),
            new TestCaseData(
                null,
                1,
                null,
                1,
                2)
                .Returns(true)
                .SetName("A02: Equals(null, 1, null, 1, 2)"),
            new TestCaseData(
                new byte[] { },
                0,
                new byte[] { },
                0,
                0)
                .Returns(true)
                .SetName("A11: Equals([], 0, [], 0, 0)"),
            new TestCaseData(
                new byte[] { 0 },
                1,
                new byte[] { 0 },
                1,
                0)
                .Returns(true)
                .SetName("A12: Equals([0], 1, [0], 1, 0)"),

            //// 比較
            new TestCaseData(
                new byte[] { 0, 1, 2 },
                0,
                new byte[] { 0, 1, 2 },
                0,
                3)
                .Returns(true)
                .SetName("B01: Equals([0, 1, 2], 0, [0, 1, 2], 0, 3)"),
            new TestCaseData(
                new byte[] { 0, 1, 2, 3 },
                1,
                new byte[] { 9, 1, 2, 8 },
                1,
                2)
                .Returns(true)
                .SetName("B02: Equals([0, 1, 2, 3], 1, [0, 1, 2, 8], 1, 2)"),
            new TestCaseData(
                new byte[] { 0, 1, 2, 3 },
                0,
                new byte[] { 9, 8, 0, 1 },
                2,
                2)
                .Returns(true)
                .SetName("B03: Equals([0, 1, 2, 3], 0, [9, 8, 0, 1], 2, 2)"),
            new TestCaseData(
                new byte[] { 0, 1, 2, 3 },
                1,
                new byte[] { 1, 2 },
                0,
                2)
                .Returns(true)
                .SetName("B04: Equals([0, 1, 2, 3], 1, [1, 2], 0, 2)"),
            new TestCaseData(
                new byte[] { 1, 2 },
                0,
                new byte[] { 0, 1, 2, 3 },
                1,
                2)
                .Returns(true)
                .SetName("B05: Equals([1, 2], 0, [0, 1, 2, 3], 1, 2)"),

            //// 比較 (値不一致)
            new TestCaseData(
                new byte[] { 0, 1, 2 },
                0,
                new byte[] { 0, 5, 2 },
                0,
                3)
                .Returns(false)
                .SetName("C01: Equals([0, 1, 2], 0, [0, 5, 2], 0, 3)"),
            new TestCaseData(
                new byte[] { 0, 1, 2, 3 },
                1,
                new byte[] { 9, 5, 2, 8 },
                1,
                2)
                .Returns(false)
                .SetName("C02: Equals([0, 1, 2, 3], 1, [0, 5, 2, 8], 1, 2)"),
            new TestCaseData(
                new byte[] { 0, 1, 2, 3 },
                0,
                new byte[] { 9, 8, 0, 5 },
                2,
                2)
                .Returns(false)
                .SetName("C03: Equals([0, 1, 2, 3], 0, [9, 8, 0, 5], 2, 2)"),
            new TestCaseData(
                new byte[] { 0, 1, 2, 3 },
                1,
                new byte[] { 5, 2 },
                0,
                2)
                .Returns(false)
                .SetName("C04: Equals([0, 1, 2, 3], 1, [5, 2], 0, 2)"),
            new TestCaseData(
                new byte[] { 1, 2 },
                0,
                new byte[] { 0, 5, 2, 3 },
                1,
                2)
                .Returns(false)
                .SetName("C05: Equals([1, 2], 0, [0, 5, 2, 3], 1, 2)"),

            //// 範囲外
            new TestCaseData(
                new byte[] { },
                1,
                new byte[] { },
                0,
                0)
                .Returns(false)
                .SetName("X01: Equals([], 1, [], 0, 0)"),
            new TestCaseData(
                new byte[] { },
                0,
                new byte[] { },
                1,
                0)
                .Returns(false)
                .SetName("X02: Equals([], 0, [], 1, 0)"),
            new TestCaseData(
                new byte[] { 0 },
                0,
                new byte[] { },
                0,
                1)
                .Returns(false)
                .SetName("X03: Equals([0], 0, [], 0, 1)"),
            new TestCaseData(
                new byte[] { },
                0,
                new byte[] { 0 },
                0,
                1)
                .Returns(false)
                .SetName("X04: Equals([], 0, [0], 0, 1)"),
            new TestCaseData(
                new byte[] { 0, 1 },
                2,
                new byte[] { 0, 1 },
                0,
                1)
                .Returns(false)
                .SetName("X05: Equals([0, 1], 2, [0, 1], 0, 1)"),
            new TestCaseData(
                new byte[] { 0, 1 },
                0,
                new byte[] { 0, 1 },
                2,
                1)
                .Returns(false)
                .SetName("X06: Equals([0, 1], 0, [0, 1], 2, 1)"),
            new TestCaseData(
                null,
                0,
                new byte[] { },
                0,
                0)
                .Returns(false)
                .SetName("X11: Equals(null, 0, [], 0, 0)"),
            new TestCaseData(
                new byte[] { },
                0,
                null,
                0,
                0)
                .Returns(false)
                .SetName("X12: Equals([], 0, null, 0, 0)"),
        };


        /// <summary>
        /// インスタンスを作成します。
        /// </summary>
        public ArrayUtilityTest()
        {
            // AVOID
        }


        /// <summary>
        /// <see cref="ArrayUtility.Equals(byte[],int,byte[],int,int)"/> をテストします。
        /// </summary>
        /// <param name="array1">配列1。</param>
        /// <param name="offset1">比較を開始する <paramref name="array1"/> の開始位置。</param>
        /// <param name="array2">配列2。</param>
        /// <param name="offset2">比較を開始する <paramref name="array2"/> の開始位置。</param>
        /// <param name="length">比較する要素数。</param>
        /// <returns>
        /// <see cref="ArrayUtility.Equals(byte[],int,byte[],int,int)"/> の戻り値。
        /// </returns>
        [Test]
        [TestCaseSource("EqualsByteTestData")]
        public bool TestEqualsByte(byte[] array1, int offset1, byte[] array2, int offset2, int length)
        {
            return ArrayUtility.Equals(array1, offset1, array2, offset2, length);
        }
    }
}